#!/usr/bin/env python
# -*- coding: UTF-8 -*-


class CSVreader():
	"""Reads the content of a CSV file and returns a dictionnary of
	lists containing its information."""

	def __init__(self,path2csvFile):
		"""Initializes the 'content' attribute."""
		self.content = self.parseCSV(path2csvFile)


	def cut(self,line):
		""" Function allowing to read the content of a line in the input
		csv file. It returns the axis considered (first field of the 
		file), the identifier of the first media (second field) and that
		 of the second one (third one)."""
		i = 0
		axis = ""
		while (i<len(line)) and (line[i]!=","):
			axis += line[i]
			i += 1
		i += 1
		med1 = ""
		while (i<len(line)) and (line[i]!=","):
			med1 += line[i]
			i += 1
		i += 1
		med2 = ""
		while (i<len(line)) and (line[i]!=","):
			med2 += line[i]
			i += 1
		i += 1
		url1 = ""
		while (i<len(line)) and (line[i]!=","):
			url1 += line[i]
			i += 1
		i += 1
		url2 = ""
		while (i<len(line)) and (line[i]!=",") and (line[i:i+1] != "\n"):
			url2 += line[i]
			i += 1
		return [ axis, int(med1), int(med2) , url1, url2 ]


	def parseCSV(self,path2file):
		"""Parse the csv file and returns its content in a dictionnary
		of lists."""
		result = {
			"axis" : [],
			"idMedia1" : [],
			"idMedia2" : [],
			"urlMedia1" : [],
			"urlMedia2" : [] }
		csvFile = open(path2file,'r')
		lineNumber = 0
		for line in csvFile:
			if (lineNumber == 0):
				lineNumber = 1  # we have to "jump over" the first line
			else:
				data = self.cut(line)
				result["axis"].append(data[0])
				result["idMedia1"].append(data[1])
				result["idMedia2"].append(data[2])
				result["urlMedia1"].append(data[3])
				result["urlMedia2"].append(data[4])
		return result


	def getComparisonData(self,rank):
		"""Returns the data corresponding to the rank-th comparison."""
		return { "axis": self.content["axis"][rank],
				"idmedia1": self.content["idMedia1"][rank],
				"idmedia2": self.content["idMedia2"][rank],
				"urlmedia1": self.content["urlMedia1"][rank],
				"urlmedia2": self.content["urlMedia2"][rank] }


	def numberOfComparisons(self):
		"""Returns the number of comparisons found in the CSV file."""
		return len(self.content["axis"])


	def testClass(self):
		"""Allows to check if this class works by printing the content
		of the 'content' attribute like a csv file."""
		print ( "\n--\ntesting the CSVreader class by parsing the "
			+ "content of '../data/0.csv' and printing it:\n" )
		print "axis,idMedia1,idMedia2,urlMedia1,urlMedia2"
		for i in range(0,len(self.content["axis"])):
			print ( str(self.content["axis"][i])
				+ "," + str(self.content["idMedia1"][i])
				+ "," + str(self.content["idMedia2"][i])
				+ "," + str(self.content["urlMedia1"][i])
				+ "," + str(self.content["urlMedia2"][i]) )
		print ( "\n--\n" + str( self.numberOfComparisons() )
			+ " comparisons found." )
		for i in range(0,2):
			comp = self.getComparisonData(i)
			print ("\n--\n" + str(i)+"th comparison asked: is the media whose id is " + str(comp["idmedia1"])
				+ " and whose url is " + comp["urlmedia1"] + " greater along an axis representing " + comp["axis"]
				+ " than the one whose id is " + str(comp["idmedia2"]) + " and whose url is " + comp["urlmedia2"] + " ?" )
		print "\neverything seems to have gone fine...\n"


# --------------- TEST

if (__name__ == "__main__"):
	cr = CSVreader("../data/HITdata/0.csv")
	cr.testClass()
