#!/usr/bin/env python
# -*- coding: UTF-8 -*-


import os
import random

class Order:
	"""Creates a random order in which to sort media, saves it and
	returns the results of a comparison along a given axis following one
	these random orders."""


	def __init__(self,elmts,axes):
		"""Creates an instance of Order that has to the content of the
		'elmts' list along the axes contained in the 'axes' list
		argument. If files containing previously created orders exists
		in ./orders they will be used."""
		self.directory = "../data/CM-stub/order/"
		self.elmts = elmts
		self.axes = axes
		self.ordered = {}
		for axis in axes:
			self.ordered[axis] = self.createOrder(axis)


	def createOrder(self,axis):
		"""Either reads the content of an order file if it exists or
		creates a file containing a new order."""
		if axis in os.listdir(self.directory):
			order = self.readOrder(axis)
		else:
			order = self.elmts
			random.shuffle(order)
			self.writeOrder(order,axis)
		return order


	def getGreatest(self,elmt1,elmt2,axis):
		"""Returns the greatest element among 'elmt1' and 'elmt2' along
		the 'axis' axis."""
		order = self.ordered[axis]
		if (order.index(elmt1) > order.index(elmt2)):
			return elmt1
		else:
			return elmt2


	def checkResults(self,results):
		"""Calculates the distance between saved orders and the results
		given as parameters."""
		dist = {}
		for axis in self.axes:
			dist[axis] = 0.0
			for i in range(0,len(results[axis])):
				dist[axis] += abs( i - self.ordered[axis].index(results[axis][i]) ) / len(results[axis])
		print dist


	def readOrder(self,path):
		"""Reads the content of the 'path' file."""
		order = []
		f = open(self.directory + path, 'r')
		for line in f:
			order.append(int(line))
		f.close()
		return order


	def writeOrder(self,order,path):
		"""Writes the content of the 'order' list in a file located in
		'path'."""
		f = open(self.directory + path, 'w')
		for x in order:
			f.write(str(x) + "\n")
		f.close()


	def testClass(self):
		"""Tests the methods of this class."""
		for axis in self.axes:
			print axis," - ==> + ",self.ordered[axis]
			print "Comparisons:"
			for x in self.elmts:
				for y in self.elmts:
					if (self.getGreatest(x,y,axis) == x):
						print "\t",x,">",y
					else:
						print "\t",x,"<",y


# --------------- TEST

if (__name__ == "__main__"):
	od = Order(range(0,4),["axe1","axe2"])
	od.testClass()

