#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# This stub emulates the behaviour of CrowdFlower's crowd : it takes
# the csv file containing media to submit to the turkers and returns
# another csv file containing the results. These results are "obtained"
# using nTurkers turkers ; each HIT submitted receives nAnnotation
# annotations. Each turker has an errRate error rate to simulate
# incorrect results.

import random
import sys
import os
import subprocess
import time




# --------------- Turkers

def generateIdTurker():
	# generates a turker's identifier
	id = ""
	for i in range(0,9):
		id += random.choice(['1','2','3','4','5','6','7','8','9','a','b','c','d','e','f'])
	return id


def errorDistribution(numTurker):
	# returns the error rate corresponding to a number in
	# range(0,nTurker). The aim is to allow to test several
	# distributions of error rates among the turkers.
	x = float(numTurker) / N_TURKERS
	if (x < 0.8):
		return 0.05
	elif (x < 0.95):
		return 0.05
	elif (x < 0.97):
		return 0.4
	else:
		return 0.5



def createTurkers():
	names = []
	errRates = []
	for i in range(0,N_TURKERS):
		names.append( generateIdTurker() )
		errRates.append( errorDistribution(i) )
	return [names, errRates ]


# --------------- I-O

def cut(line):
	# Function allowing to read the content of a line in the input csv
	# file. It returns the axis considered (first field of the file),
	# the identifier of the first media (second field) and that of the
	# second one (third one)

	i = 0
	axis = ""
	while (i<len(line)) and (line[i]!=","):
		axis += line[i]
		i += 1
	i += 1
	med1 = ""
	while (i<len(line)) and (line[i]!=","):
		med1 += line[i]
		i += 1
	i += 1
	med2 = ""
	while (i<len(line)) and (line[i]!=","):
		med2 += line[i]
		i += 1
	i += 1
	url1 = ""
	while (i<len(line)) and (line[i]!=","):
		url1 += line[i]
		i += 1
	i += 1
	url2 = ""
	while (i<len(line)) and (line[i]!=",") and (line[i:i+1] != "\n"):
		url2 += line[i]
		i += 1
	return [ axis, int(med1), int(med2) , url1, url2 ]


def getMedia():
	# Returns the list of the names of the media to sort. It uses the
	# content of the file ../data/media.txt
	mediaNames = []
	lst = open("../data/media.txt","r")
	for line in lst:
		# adding the content of the line without the ending "\n"
		mediaNames.append(line[:len(line)-1])
	lst.close()
	return mediaNames


def hitLine(param,idTurker,orders,wrong):
	# Returns the line to be added in the results.csv file corresponding
	# to its parameters. If the boolean "wrong" is true, the comparison
	# result returned is wrong
	line = "12,12:20,34,23:20,pi2/6,94.0,"+idTurker+",France,Rhone-Alpes,Ecully,127.0.0.1,"
	axis = param[0]
	idMedia1 = param[1]
	idMedia2 = param[2]
	urlMedia1 = param[3]
	urlMedia2 = param[4]
	if not wrong:
		if ( orders[axis].index(idMedia1) > orders[axis].index(idMedia2) ) :
			line+="Media 1,"
		else:
			line+="Media 2,"
	else:
		if ( orders[axis].index(idMedia1) > orders[axis].index(idMedia2) ) :
			line+="Media 2,"
		else:
			line+="Media 1,"
	line += str(idMedia1)+","+str(idMedia2)+","+axis+","+urlMedia1+","+urlMedia2
	return line


def getInput(iteration):
	# Returns the content of the input file in a string
	data = open("../data/HITdata/"+str(iteration)+".csv","r")
	return data


def generateOutput(inputData,orders,names,errRates):
	# Returns the content to be written in the HITresults folder
	out = ""
	i = 0
	for line in inputData:
		if i == 0 :
			out += "_unit_id,_created_at,_id,_started_at,_channel,_trust, _worker_id,_country,_region,_city,_ip,which_media_conveys_the_most,idMedia1,idMedia2,axis,urlmedia1,urlmedia2\n"
			i = 1
		else:
			for j in range(0,N_ANNOTATIONS):
				rank = random.randint(0,N_TURKERS-1)
				turker = names[rank]
				errLikeliness = errRates[rank]
				out += hitLine( cut(line),turker,orders, (random.random() < errLikeliness) ) + "\n"
	return out


def writeOutput(content,iteration):
	# Writes the output of the HITs in the csv file corresponding to
	# this iteration
	out = open("../data/HITresults/results"+str(iteration)+".csv","w")
	out.write(content)
	out.close()
	webhook = open("../data/webhook/go.txt","w")
	webhook.write("webhook received")
	webhook.close()
	return 0


# --------------- Order used along each axis

def obtainOrders(name):
	# Returns an array for each axis considered containing a random
	# combination of numbers between 0 and nMedia. The contents are
	# created using the createOrders function. The arrays are stored in
	# a dictionnary.
	orders = {}
	for axis in os.listdir("../data/CM-stub/" + name):
		if ( axis[0:3]!="res" and axis[len(axis)-3:]!=".py" and  axis[len(axis)-4:]!=".pyc" and axis != "log" ):
			orders[axis] = []
			content = open("../data/CM-stub/" + name + axis,"r")
			for line in content:
				orders[axis].append(int(line))
			content.close()
	return orders


def createOrders():
	# Creates a file for each axis contained in the axes.txt file and
	# containing number from 0 to nMedia in a random order.
	nMedia = len(getMedia())
	axesFile = open("../data/axes.txt","r")
	axes = []
	for line in axesFile:
		axes.append(line[:len(line)-1])
	for axis in axes:
		content = open("../data/CM-stub/"+axis,"w")
		numbers = range(0,nMedia)
		random.shuffle(numbers)
		for i in range(0,nMedia):
			content.write( str(numbers[i]) + "\n" )
		content.close()
	return 0


# --------------- Checking the results

def flushAll():
	# Removes all content to allow a new test
	os.system("""echo "DROP DATABASE DBcps ; " | mysql --user=root -p4re:bel0ng""")
	print "DBcps database is now empty"
	os.system("""mysql --user=root -p4re:bel0ng < ../data/mysql/createCPStables.sql""")
	print "New DBcps created"
	os.system("""rm -rf ../data/HITdata/* ; rm -rf ../data/HITresults/*; """)
	print "csv files removed"


def checkResults(res,orders):
	# returns the number of differences between the contents of res and
	# orders
	print ""
	for axis in res.keys():
		dist = 0
		print "\n" + axis + " : "
		for i in range(0,len(res[axis])):
		#	print str( res[axis][i] ) + "\t" + str( orders[axis][i] )
			for j in range(0,len(orders[axis])):
				if (res[axis][i] == orders[axis][j]):
					dist += abs(i-j)
		final_dist = (float(dist)) / len(res[axis])
		print "\tdistance = " + str(final_dist) +"\n"
	print ""


# --------------- Checking the program

def extendedTests():
	# Performs numberOfTests call to cps. The aim is to check wether it
	# works or not by launching it with different values for N_Turkers
	# and different orders
	os.system("""rm ../data/CM-stub/output*""")
	print "previous test results removed"
	print "\nTest launched"
	flushAll()
	#createOrders()
	logName = "../data/CM-stub/output.py"
	log = open(logName,"w")
	log.write("#===========================================================\n")
	log.write("#                         TEST\n" )
	log.write("#===========================================================\n\n\n")
	log.write("#  N_TURKERS = " + str(N_TURKERS) + "\n")
	log.write("#  N_MEDIA = " + str(len(MEDIA)) + "\n")
	log.write("#  N_ANNOTATIONS = " + str(N_ANNOTATIONS) + "\n\n\n")
	log.write("#--------- CPS OUTPUT ----------\n\n\n")
	log.write("cont = ")
	proc = subprocess.Popen("java -cp 'bin/:lib/*' myShell.MainClass --sort 'R12e^Of.The:M4ch1ne'",stdout=subprocess.PIPE,shell=True)
	log.write(proc.stdout.read())


def getResults():
	# returns the results of the last extended test
	print "interpretation :"
	o = obtainOrders("")
	res = __import__("output")
	print res.cont
	checkResults(res.cont,o)
	print "\n----\n"




# ######################################################################

if __name__ == "__main__":
# --------------- constants
	N_TURKERS = 30
	N_ANNOTATIONS = 3
	#MEDIA = getMedia()
	if (len(sys.argv) == 3 ):
# --------------- fills the media.txt file with names of media (media$i)
		if (sys.argv[1] == "newmedia"):
			mediaFile = open("../data/media.txt","w")
			for i in range(0,int(sys.argv[2])):
				mediaFile.write("media" + str(i)+"\n")
			mediaFile.close()
			print "new media.txt file written (" + str(i+1) + "entries)"
	elif (len(sys.argv) == 2 ):
# --------------- performs extended test
		if (sys.argv[1] == "test"):
			extendedTests()
# --------------- retrieves the last extended test's results
		elif (sys.argv[1] == "results"):
			getResults()
# --------------- generating the order files for each emotion
		elif (sys.argv[1] == "neworders"):
			createOrders()
			orders = obtainOrders("")
			for axis in orders.keys():
				print axis,orders[axis]
# --------------- displaying help
		elif (sys.argv[1] == "help"):
			print "\n\nCrowdManager Stub"
			print ""
			print "This script is a stub alowwing the test of the CrowdPoweredSort program without"
			print 'using a "real" crowd. Instead, it behaves like crowdFlower, i.e it takes a csv'
			print "file containing the information necessary to create the HITs as an argument"
			print "(../data/HITdata/$(number).csv) and returns another csv file containing"
			print "simulated HIT results. The error rate used for the turkers can be adjusted"
			print 'by modifying the "errorDistribution" function.'
			print ""
			print "Launching this script without any arguments will launch the creation of the"
			print "results. Make sure the files $(each axis) are present in the"
			print "../data/CM-stub/ directory. If not, call this script :"
			print "python ../data/CM-stub/CM-stub.py 'firstaxis' ... 'lastaxis'"
			print "\n\n"
# --------------- erasing results in the database and the csv files
		elif (sys.argv[1] == "flush"):
			flushAll()
# --------------- comparing a result (i.e, numbers separated by commas)
#				 with the content of the "order files"
		elif (sys.argv[1] == "compare"):
			results = {}
			refOrders = obtainOrders("")
			for axis in refOrders.keys():
				print "what where the results along the "+axis+" axis ?"
				results[axis] = input()
			checkResults(results,refOrders)
# --------------- acts as a proper stub
		else:
			iterationNumber = int(sys.argv[1])
			inputData = getInput(iterationNumber)
			refOrders = obtainOrders("")
			[ turkersId, turkersEr ] = createTurkers()
			outputData = generateOutput(inputData,refOrders,turkersId,turkersEr)
			writeOutput(outputData,iterationNumber)
