<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
	<?php
		include('docUtils.php');
		principleHeader('Components');
		principleMenu("components.php");
	?>


	<h2>Overview</h2>
	<div class="diagramm">
		<p>There are several components interacting with each other when you use CPS. They can be all on the same machine or on different ones, depending on your needs and what you have at your disposal.</p>
		<ol>
			<li>First of all is the server <strong>CPS Host</strong>, hosting CPS <em>i.e</em> the java <a href="https://secure.wikimedia.org/wikipedia/en/wiki/Bytecode">bytecode</a> (the content of the <strong>bin</strong> directory. It is in charge of downloading jobs results, parsing them, using them for a splitsort iteration and then uploading a new job containing new comparisons.</li>
			<li>
				<p>When the <strong>workers</strong> have answered all of the questions, <strong>CrowdFlower</strong> sends a <a href="https://crowdflower.com/docs/api/webhooks/">webhook</a>. But instead of having it sent directly to you, it first arrives in the <strong>Antechamber</strong>. This component consists in several PHP scripts triggered by the webhook. Their aim is to send you an email when a job is finished, therefore filtering these webhooks (there can be up to tens of thousands of <a href="https://crowdflower.com/docs/api/units/"> units</a> per job, the end of each being spotted. Your webmail may <em>not</em> appreciate such a flood).</p>
				<p>Furthermore, since CrowdFlower decided unilateraly (and without leaving the slightest warning for the users) to stop supporting the downloading of results as a CSV files, the <strong>Antechamber</strong> earned a new role : it parses the <a href="https://secure.wikimedia.org/wikipedia/en/wiki/JSON">JSON</a> formatted results and returns a csv file in order for CPS to work without a huge re-writing. The mail is sent <em>after</em> this operation.</p></li>
			<li>Since you must display the media you want to sort for the workers to compare them, a server, <strong>Media Host</strong> must host all of them.</li>
			<li>At last, <a href="https://www.crowdflower.com/"><strong>CrowdFlower</strong></a> and its <strong>workers</strong> also interacts with these components, though you cannot host them where you please.</li>
		</ol>
		<p>Here is a scheme that summarizes how these elements interact with each other. An array symbolises information flux.</p>
		<div align="center"><img src="figures/components.png" height="500" width="710" alt="a scheme describing how the different components interact"></div>
	</div>


	<h2>CPS Host</h2>
	<div class="diagramm">
		<p>This server host the CPS program, <em>i.e</em> the one whose documentation you are browsing right now. Since what this program does is discussed in pretty much every where else in this website, it will not be detailed here. However, in order for this to work as well as possible, here is what the server <strong>must</strong> have:</p>
		<ul>
			<li>A <a href="https://secure.wikimedia.org/wikipedia/en/wiki/Mysql">MySQL</a> database.</li>
			<li>An Internet access allowing it to download and upload data from the <strong>antechamber</strong>. The bandwidth does not have to be huge because only text files of several Mo will be exchanged.</li>
			<li>A Java Virtual Machine (<a href="https://secure.wikimedia.org/wikipedia/en/wiki/Java_Virtual_Machine">JVM</a>).</li>
			<li>At least a few Go of free space.</li>
		</ul>
		<p>It also <strong>should</strong> have as much computing power as possible. Indeed, the <a href="http://code.google.com/p/get-another-label/">get another label</a> program, which is called in order to increase the quality of the data, requires a lot of calculations.</p>
	</div>


	<h2>Antechamber</h2>
	<div class="diagramm">
		<p>As its name indicates it, this server is in charge of receiving <a href="https://crowdflower.com/docs/api/webhooks/">webhook</a> and preparing it to be sent to you. Depending on their <tt>signal</tt> attibute, the webhooks will trigger the sending of an e-mail to the user after parsing their <tt>payload</tt> attribute (the one alleged to contain the job results). This server <strong>must</strong> have:</p>
		<ul>
			<li>An Internet connection. Well, you know, the <a href="https://secure.wikimedia.org/wikipedia/en/wiki/Http"><tt>http</tt></a> protocol, used by the webooks, <em>demands</em> it.</li>
			<li>A <a href="https://secure.wikimedia.org/wikipedia/en/wiki/Mail_server">mail server</a>. This might be a little tickier to obtain if you want to install it yourself.</li>
			<li>An <a href="https://secure.wikimedia.org/wikipedia/en/wiki/Apache_HTTP_Server">Apache</a> server configured to be able to run <a href="https://secure.wikimedia.org/wikipedia/en/wiki/PHP">PHP <!--aka the worst language ever --></a> scripts.</li>
		</ul>
		<p>Since it will most likely send less than <em>one e-mail per day</em> and parse a several Mo big JSON file as rarely, there is no need for the father of all servers here. The simplest web server you can get should be just fine. PHP scripts you can use as well as more details on how this component works are given a <a href="antechamber.php">dedicated section</a>.</p>
	</div>


	<h2>Media Host</h2>
	<div class="diagramm">
		<p>This server will host the media you want to sort, <em>i.e</em> the "real" media, not only their names like in the <a href="../userManual/xmlFiles.php"><tt>media.xml</tt></a> file. The HITs the workers will download will contain media: these have to be stored somewhere! Here's what this server <strong>must</strong> have:</p>
		<ul>
			<li>An internet connection (again) allowing distant users to download its content. Indeed, the media it contains will most likely not travel by homing pigeon.</li>
			<li>A web server, like <a href="https://secure.wikimedia.org/wikipedia/en/wiki/Apache_HTTP_Server">Apache</a>, to allow distant users to connect with it.</li>
		</ul>
		<p>Depending on the type of media you want to use, their may be a need for a rather high bandwidth and/or computing power, for example if you want to sort videos.</p>
	</div>


	<?php
		printFooter();
	?>
</body>
</html>
